function bpX = apply_filterbank(x, config, filterbank, fs)
  %% bpX = apply_filterbank(x, config, filterbank, fs)
  % Applies a filterbank provided by the argument filterbank to the input
  % signal x.
  %
  % x: input signal, may be mono or multi channel.
  %
  % config: the property config.groupDelayCompensation defines how the
  % group delay of the filter bank will be compensated. Possible values are
  % 'max' for complete group delay compensation, 'mean' for compensating up
  % to the average group delay of the filter bank, 'none' for no group
  % compensation at all or an explicit number in milliseconds that will
  % define the maximum compensation.
  %
  % filterbank: this struct holds all information of the filterbank.
  % filterbank.Hd is an array with bandpass filters, filterbank.nBands is
  % the number of allpass filters, and filterbank.centerFrequencies
  % contains the center frequency of every filter in Hd.
  %
  % fs: sampling rate of the input signal and centerFrequencies.
  Hd = filterbank.Hd;
  nBands = filterbank.nBands;
  bpX = zeros([size(x, 1), size(x, 2), nBands]);
  
  
  %% calculate group delay
  fprintf('\ncalculating group delay\n');
  tic
  groupDelay = calcGroupDelay(filterbank.Hd, filterbank.centerFrequencies, fs);
  toc
  
  if strcmpi(config.groupDelayCompensation, 'max')
    maxDelay = round(max(groupDelay));
  elseif strcmpi(config.groupDelayCompensation, 'mean')
    maxDelay = round(mean(groupDelay));
  elseif strcmpi(config.groupDelayCompensation, 'none')
    maxDelay = 0;
  elseif isnumeric(config.groupDelayCompensation)
    maxDelay = round(config.groupDelayCompensation*fs/1000);
  else
    disp('invalid parameter for group delay. setting maxdelay to 0 (no compensation)');
    maxDelay = 0;
  end
  
  groupDelay = floor(groupDelay);
  shiftDelay = maxDelay - groupDelay;
  shiftDelay(shiftDelay < 0) = 0;
  fprintf('chosen delay is %d ms. maximum group delay is %d ms, average group delay is %d ms\n', ...
        round(maxDelay/fs*1000), round(max(groupDelay)/fs*1000), round(mean(groupDelay)/fs*1000));
  
  %% apply bandpass filter to audio signal and pitch shift
  tic
  fprintf('\napplying bandpass filter to audio signal\n');
  parfor i=1:nBands
    x_bp = filter(Hd(i), x);
    if shiftDelay(i) > 0 % compensate group delay of bandpass filter
      x_bp = shiftd(x_bp, 0, shiftDelay(i));
    end
    bpX(:,:,i) = x_bp;
  end
  toc
end