function [Hd, nBands, centerFrequencies] = design_filterbank_fir(fBase, fs, BandsPerOctave, filterDistribution, stopbandAttenuation)
  %% [Hd, nBands, centerFrequencies] = design_filterbank_fir(fBase, fs, BandsPerOctave, filterDistribution, stopbandAttenuation)
  % Designs an FIR filter bank using different design criteria and a
  % Kaiser window filter design.
  %
  % fBase: base frequency for filter bank.
  %
  % fs: sampling rate
  %
  % BandsPerOctave: number of band-pass filters per octave. This property
  % is ignored if filterDistribution is set to linear.
  %
  % filterDistribution: 'log' or 'lin' for logarithmic or linear
  % distribution. Default is 'log'. If set to 'lin', the parameter
  % BandsPerOctave is ignored, and the spacing of the filters will be equal
  % to the base frequency fBase.
  %
  % stopbandAttenuation: defines the stopbandAttenuation of the FIR Kaiser
  % band-pass filters in dB.
  %
  %
  % Returns an array Hd of size nBands that contains the bandpass filter
  % objects. Also returns an array centerFrequencies that contains the
  % center frequencies for each filter in Hd.

  % Derivation for nBands:
  %
  % fBase * 2.^(i/12) < fs/2
  % log_2(fBase) + i/12 < log2(fs/2)
  % i < 12*log2(fs/2) - log2(fBase)*12

  if strcmpi(filterDistribution, 'lin')
    nBands = floor((fs/2)/fBase);
    centerFrequencies = fBase*(1:nBands);
  else
    nBands = floor(BandsPerOctave*log2(fs/2) - log2(fBase)*BandsPerOctave);
    centerFrequencies = fBase * 2.^( (1:nBands) /BandsPerOctave);
  end
  
  normLFreq = centerFrequencies(1:end-2) / (fs/2); % center frequencies of the previous (left) band
  normCFreq = centerFrequencies(2:end-1) / (fs/2); % center frequencies of the current band
  normRFreq = centerFrequencies(3:end) / (fs/2); % center frequencies of the next (right) band
  
  centerFrequencies = centerFrequencies(2:nBands-1);
  nBands = nBands - 2;
  
  Hd(nBands) = fdesign.bandpass;
  parfor i=1:nBands
    fs1 = normLFreq(i);
    fs2 = normRFreq(i);
    df1 = normCFreq(i) - fs1;
    df2 = fs2 - normCFreq(i) ;
    fp1 = normCFreq(i)-df1/100;
    fp2 = normCFreq(i)+df2/100;
    d = fdesign.bandpass(fs1, fp1, fp2, fs2, stopbandAttenuation, 1, stopbandAttenuation);
    Hd(i) = design(d,'kaiserwin');
  end
end