function [Hd, nBands, centerFrequencies] = design_filterbank_iir(fBase, fs, BandsPerOctave, filterDistribution, stopbandAttenuation)
  %% [Hd, nBands, centerFrequencies] = design_filterbank_iir(fBase, fs, BandsPerOctave, filterDistribution, stopbandAttenuation)
  % Designs an IIR filter bank using different design criteria and a
  % Butterworth filter design.
  %
  % fBase: base frequency for filter bank.
  %
  % fs: sampling rate
  %
  % BandsPerOctave: number of band-pass filters per octave. This property
  % is ignored if filterDistribution is set to linear.
  %
  % filterDistribution: 'log' or 'lin' for logarithmic or linear
  % distribution. Default is 'log'. If set to 'lin', the parameter
  % BandsPerOctave is ignored, and the spacing of the filters will be equal
  % to the base frequency fBase.
  %
  % stopbandAttenuation: defines the stopbandAttenuation of the IIR
  % Butterworth band-pass filters in dB.
  %
  %
  % Returns an array Hd of size nBands that contains the bandpass filter
  % objects. Also returns an array centerFrequencies that contains the
  % center frequencies for each filter in Hd.

  % Derivation for nBands:
  %
  % fBase * 2.^(i/12) < fs/2
  % log_2(fBase) + i/12 < log2(fs/2)
  % i < 12*log2(fs/2) - log2(fBase)*12

  if strcmpi(filterDistribution, 'lin')
    nBands = floor((fs/2 - fBase)/fBase); % FIXME: do I need to subtract -fBase or does it work without that term?
    centerFrequencies = fBase*(1:nBands);
  else
    nBands = floor(BandsPerOctave*log2(fs/2) - log2(fBase)*BandsPerOctave);
    centerFrequencies = fBase * 2.^( (1:nBands) /BandsPerOctave);
  end
  
  normCFreq = centerFrequencies / (fs/2);
  
  for i=2:nBands-1
    fs1 = normCFreq(i-1);
    fs2 = normCFreq(i+1);
    df1 = normCFreq(i) - fs1;
    df2 = fs2 - normCFreq(i) ;
    fp1 = normCFreq(i)-df1/100;
    fp2 = normCFreq(i)+df2/100;
    d = fdesign.bandpass(fs1, fp1, fp2, fs2, stopbandAttenuation, 1, stopbandAttenuation);
    Hd(i-1) = design(d,'butter');
  end
  
  centerFrequencies = centerFrequencies(2:nBands-1);
  nBands = nBands - 2;
end