function y = filterbank_waveshaping(bpX, waveshaper, mode)
  %% y = filterbank_waveshaping(bpX, waveshaper, mode)
  % Applies a waveshaping algorithm to every band in the 4-dimensional
  % array bpX, and returns the combined (summed) signal y.
  %
  % bpX: four-dimensional array of size
  % nSamples * nChannels * nBandsPerOctave * nPitchBands
  %
  % waveshaper: function to apply waveshaping. Possible waveshapers are
  % filter8bit(x) or square_waveshaping_analog(x).
  %
  % mode (optional): If set to 'sum', the bands of each pitch are summed
  % before being sent to the waveshaper. By default, each band is processed
  % individually. Note that when set to 'sum', waveshaping will not work
  % well with polyphonic signals.

  %% apply waveshaping to audio signal
  if nargin >= 2 && ~isempty(waveshaper)
    tic
    fprintf('\napplying waveshaping to audio signal\n');
    BandsPerOctave = size(bpX, 3);
    nPitchBands = size(bpX, 4);
    
    if nargin >= 3 && strcmpi(mode, 'sum')
      bpX = sum(bpX, 3);
      parfor j=1:nPitchBands
        bpX(:,:,:,j) = waveshaper( bpX(:,:,:,j) );
      end
    else
      parfor k=1:BandsPerOctave
        for j=1:nPitchBands
          bpX(:,:,k,j) = waveshaper( bpX(:,:,k,j) );
        end
      end
    end
    
    toc
  else
    disp('no waveshaper provided. just merging the audio signal');
  end
  
  if ~nargout || nargin == 3
    y2 = squeeze(bpX);
    figure;
    plot(y2(:,:,1));
  end
  
  if nargout
    %% Return the sum of bandpassed signals
    y = sum(sum(bpX, 4), 3);
    % normalize
    if(max(max(y)) >= 1)
      disp('NOTE (from waveshaper): output clips. normalizing.');
      y = y/max(max(y))*0.9999;
    end
  end
end